-- RADAR CMU Website Schema
-- written for postgres
-- Anthony Tomasic 1/12/2005

DROP SCHEMA www CASCADE;
CREATE SCHEMA www;

-- projectrole
-- the list of project roles for RADAR
CREATE TABLE www.projectrole (
	id SERIAL PRIMARY KEY,
	caption TEXT
);

-- person_projectrole
-- associations of people to projectroles for the site
CREATE TABLE www.person_projectrole (
	person_id INTEGER,
	projectrole_id INTEGER
);

-- positiontype
-- the types of positions for people on the website
CREATE TABLE www.positiontype (
	id SERIAL PRIMARY KEY,
	caption TEXT
);

-- person_positiontype
-- links from people to their type of position
CREATE TABLE www.person_positiontype (
	person_id INTEGER,
	positiontype_id INTEGER
);

-- position
-- the list of positions of people
CREATE TABLE www.position (
	id SERIAL PRIMARY KEY,
	caption TEXT
);

-- person_position
CREATE TABLE www.person_position (
	person_id INTEGER,
--	FOREIGN KEY (person_id) REFERENCES person(id),
	position_id INTEGER
);

-- category manages the display of things on the website
CREATE TABLE www.category (
	id SERIAL PRIMARY KEY,
	-- parent INTEGER,
	parent TEXT,
	caption TEXT,
	distribution TEXT,	-- security "external", "darpa", "internal"
	renderer TEXT,	-- the function to call, can be empty?
	collapse TEXT,	-- '1' ?
	hidden TEXT,	-- ?
	showname TEXT 	-- '1' ?
);

CREATE TABLE www.categorylink (
	category_id INTEGER,
	item_id INTEGER
);

-- event type manages the display of events on the website
CREATE TABLE www.eventtype (
	id SERIAL PRIMARY KEY,
	name TEXT,	-- lowercase name
	caption TEXT -- display name
);

-- formgroup - each tuple describes a set of forms
CREATE TABLE www.formgroup (
	id	SERIAL PRIMARY KEY,
	name	TEXT,
	caption TEXT,
	sitehref TEXT
);

-- form - each tuple corresponds to a form to edit in other relations
-- see also the field relationship
CREATE TABLE www.form (
	id	SERIAL PRIMARY KEY,
	formgroup_id	INTEGER	NOT NULL,	-- group that this form belongs to
	caption		TEXT	NOT NULL,	-- title of form on html page
	key		TEXT	NOT NULL,	-- SQL to fetch values
	position	INTEGER NOT NULL,	-- position to sort in list of forms
	tablename	TEXT	NOT NULL,	-- relation name to modify
	previewurl	TEXT	DEFAULT null,	-- url to radar.jsp with appropriate 
						-- parameters to show preview page [Note: 
						-- jsp has to fill in eid to complete url]
	showtoplevel	BOOLEAN	DEFAULT	true	-- determines if this form should appear in a top level listing of all forms
);

CREATE TABLE www.field (
	id	SERIAL PRIMARY KEY,	-- field identifer
	form_id	INTEGER NOT NULL,	-- form identifier for this field (id, formid) is key
	name	TEXT NOT NULL,	-- the internal name of field in form as POST
	caption	TEXT NOT NULL,	-- the text visable caption of field
	required	BOOLEAN DEFAULT FALSE,	-- require a value
	position	INTEGER NOT NULL,	-- order of the field among these fields
	entrytype	TEXT	NOT NULL,	-- field type eg field, dropdown, multi-combo
	subform	TEXT DEFAULT NULL,	-- identifier of joined form
	entryquery	TEXT	DEFAULT NULL,	-- query used to populate list if not field 
						-- (ie dropdown or multi-combo)
	linktablename	TEXT	DEFAULT NULL,	-- if multi-combo type, name of link table 								
						-- used to link records
	linktableid	TEXT	DEFAULT NULL,	-- name of column that performs as id column 
						-- of link table
	linktablevalue	TEXT	DEFAULT NULL,	-- name of column that performs as value 
						-- column of link table. eg if it is a 
						-- author field in publication form, then 									  -- linktableid = pubid, 
						-- linktablevalue = author
	linktableorder	TEXT DEFAULT NULL,	-- column for order if an ordered MC (else null)
	linktableparam	TEXT DEFAULT NULL,	-- column for extra param for this MC (if any)
	domain	TEXT		-- domain of this field for WbE (n/a for multi-combo)
);

-- url
-- urls are first order object on website and correspond to 
-- external references of things, including software, etc.
CREATE TABLE www.url (
	 id SERIAL PRIMARY KEY,
	 href TEXT,		-- both relative and absolute
	 caption TEXT,		-- the browser caption of the link
	 description TEXT,	-- a longer description of object
     category text   -- multi-combo field
);

-- gathering_url
-- urls associated with a gathering
CREATE TABLE www.gathering_url (
	gathering_id INTEGER,
	url_id INTEGER
);

-- category_url
-- urls associated with a given page
CREATE TABLE www.category_url (
	category_id INTEGER,
	url_id INTEGER
);

-- files
-- files on the website, uploaded through the forms or through email attachments
-- to the VIO agent.

CREATE TABLE www.file (
	id SERIAL PRIMARY KEY,
	name TEXT, -- e.g. 'IETPres2005-06-12.ppt'
	type TEXT, -- e.g. 'pdf', 'ppt', 'zip', etc...
	mimetype TEXT, -- e.g. 'application/x-pdf', etc...
	role_id INT DEFAULT 1, -- www.role required for viewing this file. public=1
	lo_content oid -- large object id holding the bytes!
);

CREATE TABLE www.enumfield (          --== represents elements of enumerated types
  id SERIAL PRIMARY KEY,
  enumtype_id INTEGER NOT NULL,  -- foreign key to join element to its enumerated type
  caption TEXT NOT NULL,         -- visible name for a particular element in an enumeration
  value TEXT DEFAULT NULL,       -- hidden representation for a particular element in an enumeration, not directly used by forms
  position INTEGER NOT NULL      -- for ordering of the elements in the drop down
);
CREATE TABLE www.enumtype (      --== represents enumerated types
  id SERIAL PRIMARY KEY,
  name TEXT NOT NULL             -- name for the class of values that defines an enumerated type
);


-- roles on the website. access levels for various content
CREATE TABLE www.role (
	id SERIAL PRIMARY KEY,
	name TEXT, -- e.g. 'Internal'
	role TEXT  -- e.g. 'internal'
);

-- documents
-- metadata for documents for meetings.  linked to meetings through 
-- www.gathering_document,  linked to files through www.document_file
CREATE TABLE www.document (
	id SERIAL PRIMARY KEY,
	caption TEXT, -- short caption for website display
	title TEXT, -- longer title
	abstract TEXT, -- abstract
	author TEXT -- could be a multicombo, but not all authors are in the db
);

-- document - meeting links
CREATE TABLE www.gathering_document ( 
	gathering_id INTEGER, 
	document_id	INTEGER
);

-- file - document links
CREATE TABLE www.document_file (
	document_id INTEGER,
	file_id INTEGER
);

-- file - publication links
CREATE TABLE www.publication_file (
	publication_id INTEGER, -- www.publication.id
	file_id INTEGER
);

-- document - category (page) links
CREATE TABLE www.category_document (
	category_id INTEGER,
	document_id INTEGER
);

-- breaking news of website
CREATE TABLE www.news(
	id SERIAL PRIMARY KEY,
	person_id INTEGER,	-- author of news
	header TEXT,	-- headline of news
	body TEXT,	-- body of news
	newsdate DATE	-- date of news
);

-- project
-- a project is anything that is funded (internal, external, etc.)
CREATE TABLE www.project (
	id SERIAL PRIMARY KEY,
	caption TEXT
);

-- person_project multivalue relationship between person and project
-- status indicates relationship of person to project
CREATE TABLE www.person_project (
	person_id INTEGER,
	project_id INTEGER,
	status TEXT	-- member, leader, observer
);

-- cohort
-- a cohort is a "group", but I can't use "group" - its a keyword
CREATE TABLE www.cohort (
	id SERIAL PRIMARY KEY,
	caption TEXT,
	title TEXT
);

-- cohort_person
-- the set of people in a cohort
CREATE TABLE www.cohort_person (
	person_id INTEGER,
	cohort_id INTEGER
);

-- publication
-- a document that was published somehow - really need BIBTEX model here FIXME
CREATE TABLE www.publication (
	id SERIAL PRIMARY KEY,
	title TEXT,
	abstract TEXT,
	publicationdate DATE,
	journal TEXT,
	pages TEXT	-- xx-xx format
);

-- publication_person is multivalue relation between publication to person
CREATE TABLE www.publication_person (
	 publication_id INTEGER,
	 person_id INTEGER
);

-- publication_url links publication to url of publication
CREATE TABLE www.publication_url (
	 publication_id INTEGER,
	 url_id INTEGER
);

-- text
-- html that appears directly on the website
CREATE TABLE www.text (
	id SERIAL PRIMARY KEY,
	html TEXT
);

-- page_text
-- links from pages to the www.text items relevant to them
CREATE TABLE www.page_text (
	page_id INTEGER,
	text_id INTEGER
);

-- person_location
CREATE TABLE www.person_location (
	id SERIAL PRIMARY KEY,
	person_id INTEGER,
	location_id INTEGER,
	type TEXT	-- 'office', 'home', 'lab'
);

-- gathering_publication
-- N-to-1 publications related to an gathering
-- (we don't record "tours" of multiple gatherings for a publication)
CREATE TABLE www.gathering_publication (
	gathering_id INTEGER,
	publication_id INTEGER
);

-- presentation
CREATE TABLE www.presentation (
	id SERIAL PRIMARY KEY,
	gathering_id INTEGER, -- where else would you give a presentation?
	speaker INTEGER,	-- person_id for joining with radar.person
	title text,
	abstract text,
	host INTEGER, -- person_id for joining with radar.person. the person
				  -- hosting the speaker
	scheduler INTEGER -- person_id for joining with radar.person
					  -- this field is for the person who will be scheduling
					  -- the speaker's time during a visit
);

-- seminar
--     added for cmu.seminar test website

CREATE TABLE www.seminar (
	id SERIAL PRIMARY KEY, --unique identifier for the record
	title text,  --title of the seminar
	abstract text,  --abstract of the seminar
	building text,  --building it is held in
	room text,    --room that it is held in
	day text,  --day that seminar is being held
	month text,  --month that seminar is being held on
	year text,  --year of seminar
	starttime text,  --time for start of seminar
	endtime text   --end time for seminar
);

CREATE TABLE www.seminar_person (
	seminar_id INTEGER,
	person_id INTEGER
);
