package airspaceGUI.base;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.JTextComponent;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import org.hibernate.Session;

import airspaceGUI.expansion.FindWindow;
import airspaceGUI.expansion.MenuBarCreation;
import airspaceGUI.expansion.TabbedPanelMenu;

/**
 * This is the base for all panels in the airspaceGUI extension to manage all of
 * the base project. It provides a generic base for the panels, with common menu
 * listeners and buttons that are the same across the panels. All the subclasses
 * need to do is set the names of the buttons and the text of the status label
 * 
 * @author srbennet
 * 
 */
public abstract class BasePanel extends SubPanel implements MenuBarCreation {
	/**
	 * A window used to find specific data-containing objects
	 */
	protected FindWindow findWindow;

	/**
	 * Go to the next piece of data
	 */
	protected JMenuItem next;

	/**
	 * Go to the previous piece of data
	 */
	protected JMenuItem prev;

	/**
	 * Menu item to initiate paste operations
	 */
	protected JMenuItem paste;

	/**
	 * Menu item to initiate undo operations
	 */
	protected JMenuItem undoMenu = new JMenuItem("Undo");

	/**
	 * Menu item to initiate redo operations
	 */
	protected JMenuItem redoMenu = new JMenuItem("Redo");

	/**
	 * Button to go to the previous piece of data
	 */
	protected JButton prevButton = new JButton("Previous");

	/**
	 * Button to create a new piece of data
	 */
	protected JButton newButton = new JButton("New");

	/**
	 * Button to go to the next piece fo data
	 */
	protected JButton nextButton = new JButton("Next");

	/**
	 * Button to delete the piece of data being edited
	 */
	protected JButton deleteButton = new JButton("Delete");

	/**
	 * The label telling the user what piece of data they are on and how many
	 * there are
	 */
	protected JLabel statusLabel = new JLabel();

	// code to be able to register events and reverse the changes
	/**
	 * Manager of the undo and redo edits
	 */
	protected UndoManager undoManager = new UndoManager();

	/**
	 * The stack of undo edits to potentially undo
	 */
	protected UndoAction undoAction = new UndoAction();

	/**
	 * The stack of redo edits to potentially redo
	 */
	protected RedoAction redoAction = undoAction
			.getRedoAction(undoManager);
	
	/**
	 * if we are editing a new instance
	 * */
	protected boolean editingNew;
	
	/**
	 * Listener to add to the undoable edits stack.
	 */
	protected UndoableEditListener undoableEditListener = new UndoableEditListener() {
		public void undoableEditHappened(final UndoableEditEvent e) {
			// Remember the edit and update the menus
			undoManager.addEdit(e.getEdit());
			undoAction.updateUndoState();
			redoAction.updateRedoState();
			undoMenu.setEnabled(true);
		}
	};

	// implement listeners for the various components

	/**
	 * this document listener monitors the changes in the text fields
	 * */
	protected DocumentListener textFieldModificationListener = new DocumentListener() {
		public void changedUpdate(DocumentEvent e) {
			if (!isPageLoading()) {
				setModified(true);
			}
		}

		public void insertUpdate(DocumentEvent e) {
			if (!isPageLoading()) {
				setModified(true);
			}
		}

		public void removeUpdate(DocumentEvent e) {
			if (!isPageLoading()) {
				setModified(true);
			}
		}
	};

	/**
	 * Used to detect if popup/combo boxes have been motified
	 */
	protected ItemListener comboBoxModificationListener = new ItemListener() {
		public void itemStateChanged(final ItemEvent e) {
			if (!isPageLoading()) {
				setModified(true);
			}
		}
	};

	/**
	 * Keeps track of the state, records when it has changed, and therefore
	 * needs to be saved
	 */
	protected ChangeListener checkBoxModificationListener = new ChangeListener() {
		public void stateChanged(final ChangeEvent e) {
			if (!isPageLoading()) {
				setModified(true);
			}
		}
	};

	/**
	 * A listener to attach to save menu items or buttons to initiate the save
	 * action
	 */
	protected ActionListener saveListener = new ActionListener() {

		public void actionPerformed(final ActionEvent arg0) {
			save();
			
			beginNewTransaction();
		}

	};

	/**
	 * This initiates an undoManager of the last action undertaken
	 */
	protected ActionListener undoListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			try {
				if (undoManager.canUndo()) {
					undoManager.undo();
					redoMenu.setEnabled(true);
				}

				if (undoManager.canUndo()) {
					undoMenu.setEnabled(true);
				} else {
					undoMenu.setEnabled(false);
				}

			} catch (CannotUndoException ex) {
				Toolkit.getDefaultToolkit().beep();
				ex.printStackTrace();
			}
		}
	};

	/**
	 * This re-does the action that was undone by an undoManager action.
	 */
	protected ActionListener redoListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			try {
				if (undoManager.canRedo()) {
					undoManager.redo();
				}
				undoMenu.setEnabled(true);
				if (undoManager.canRedo()) {
					redoMenu.setEnabled(true);
				} else {
					redoMenu.setEnabled(false);
				}
			} catch (CannotUndoException ex) {
				ex.printStackTrace();
			}
		}
	};

	/**
	 * A listener to attach to the cut menu item
	 */
	protected ActionListener cutListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			// System.out.println(e);
			if (getHasFocus() instanceof JTextComponent) {
				((JTextComponent) getHasFocus()).cut();
				paste.setEnabled(true);
			}
		}
	};

	/**
	 * A listener to attach to the copy menu item
	 */
	protected ActionListener copyListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			// System.out.println(e);
			if (getHasFocus() instanceof JTextComponent) {
				((JTextComponent) getHasFocus()).copy();
				paste.setEnabled(true);
			}
		}
	};

	/**
	 * A listener to attach to the paste menu item
	 */
	protected ActionListener pasteListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			// System.out.println(e);
			if (getHasFocus() instanceof JTextComponent) {
				((JTextComponent) getHasFocus()).paste();
			}
		}
	};

	/*
	 * uncomment if there is a shared find button that needs an actionlistener
	 * protected transient final ActionListener findListener = new
	 * ActionListener() {
	 * 
	 * public void actionPerformed(ActionEvent arg0) { if
	 * (findWindow.isVisible()) { findWindow.requestFocusInWindow(); } else {
	 * findWindow.setVisible(true); } } };
	 */

	/**
	 * A listener to initiate moving to the next object
	 */
	protected ActionListener nextListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			// flag if the user chooses to stay at currect record
			boolean stay = false;

			// save the current message in case something has been edited
			if (isModified()) {
				int option = JOptionPane.showConfirmDialog(null,
						"You have made some changes to the current record. "
								+ "Do you want to save them?", "",
						JOptionPane.YES_NO_CANCEL_OPTION);
				if (option == JOptionPane.YES_OPTION) {
					save();
				} else if (option == JOptionPane.NO_OPTION) {
					setModified(false);
				} else if (option == JOptionPane.CANCEL_OPTION) {
					stay = true;
				}
			}

			if (!stay) {
				if (editingNew) {
					getDataList().remove(getIndexOfDisplayed());
				}
				
				setPageLoading(true);
				gotoNumber(getIndexOfDisplayed() + 1);
				// isModified = false;
				// setStatus(getIndexOfDisplayed());
				setButtonPermissions();
				setPageLoading(false);
			}
		}
	};

	/**
	 * A listener to initiate moving to the previous object
	 */
	protected ActionListener prevListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			// flag if the user chooses to stay at currect record
			boolean stay = false;

			// save the current message in case something has been edited
			if (isModified()) {
				int option = JOptionPane.showConfirmDialog(null,
						"You have made some changes to the current record. "
								+ "Do you want to save them?", "",
						JOptionPane.YES_NO_CANCEL_OPTION);
				if (option == JOptionPane.YES_OPTION) {
					save();
				} else if (option == JOptionPane.NO_OPTION) {
					setModified(false);
				} else if (option == JOptionPane.CANCEL_OPTION) {
					stay = true;
				}
			}

			if (!stay) {
				if (editingNew) {
					getDataList().remove(getIndexOfDisplayed());
				}
				
				setPageLoading(true);
				gotoNumber(getIndexOfDisplayed() - 1);
				// isModified = false;
				// setStatus(getIndexOfDisplayed());
				setButtonPermissions();
				setPageLoading(false);
			}
		}
	};

	/**
	 * A listener to initiate making a new object
	 */
	protected ActionListener newListener = new ActionListener() {

		public void actionPerformed(final ActionEvent arg0) {
			boolean stay = false;

			setPageLoading(true);

			if (isModified()) {
				int option = JOptionPane.showConfirmDialog(null,
						"You have made some changes to the current record. "
								+ "Do you want to save them?", "",
						JOptionPane.YES_NO_CANCEL_OPTION);

				if (option == JOptionPane.YES_OPTION) {
					save();
				} else if (option == JOptionPane.NO_OPTION) {
					setModified(false);
				} else if (option == JOptionPane.CANCEL_OPTION) {
					stay = true;
				}
			}

			if (!stay) {
				editingNew = true;
				createNewInstance();
				setStatus(getIndexOfDisplayed());
				setButtonPermissions();
			}
			setPageLoading(false);
		}

	};

	/**
	 * The eventlistener used to delete the currently shown data
	 */
	protected ActionListener deleteListener = new ActionListener() {

		public void actionPerformed(final ActionEvent arg0) {
			setPageLoading(true);

			// System.out.println("Deleting");
			int n = JOptionPane
					.showConfirmDialog(
							null,
							"Warning: All deletes are final. \n Are you sure you wish to delete this object?");
			// 1 is no, 0 is yes, 2 is cancel

			if (n == 0) {
				deleteDisplayedObject();
				beginNewTransaction();
			}

			setPageLoading(false);
		}

	};

	/**
	 * Default constructor, doesn't do much of anything
	 * 
	 */
	protected BasePanel() {
		super();

		prevButton.setName("prev");
		nextButton.setName("next");
		newButton.setName("new");
		deleteButton.setName("delete");
	}
	
	/* (non-Javadoc)
	 * @see airspaceGUI.base.SubPanel#setPageLoading(boolean)
	 */
	public void setPageLoading(boolean pageLoading) {
		super.setPageLoading(pageLoading);
		
		if (pageLoading) {
			editingNew = false;
		}
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.MenuBarCreation#save()
	 */
	public boolean save() {
		// TODO Auto-generated method stub
		return false;
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.PanelUtils#fillInText(org.hibernate.Session)
	 */
	public void fillInText(Session session) {
		// TODO Auto-generated method stub
		
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.PanelUtils#getPane()
	 */
	public JPanel getPane() {
		// TODO Auto-generated method stub
		return null;
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.PanelUtils#getTitle()
	 */
	public String getTitle() {
		// TODO Auto-generated method stub
		return null;
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.PanelUtils#gotoNumber(int)
	 */
	public void gotoNumber(int number) {
		// TODO Auto-generated method stub
		
	}



	/* (non-Javadoc)
	 * @see airspaceGUI.expansion.PanelUtils#load(org.hibernate.Session)
	 */
	public boolean load(Session session) {
		// TODO Auto-generated method stub
		return false;
	}



	/**
	 * This adds listeners for modifications to fields and values on the GUI so
	 * the modification state can be tracked
	 * 
	 * @param docModificationListener
	 *            listener to the modification on text fields
	 * @param comboBoxModificationListener
	 *            Listener to register modifications on combo boxes
	 * @param checkBoxModificationListener
	 *            Listener to register modifications on check boxes
	 */
	abstract protected void addModificationListeners(
			DocumentListener docModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener);

	/**
	 * Adds Undoable edit listeners to the fields on the GUI so that changes can
	 * be undone.
	 * 
	 * @param undoListener
	 *            The Event Handler that handles undo events
	 */
	abstract protected void addUndoableEditListeners(
			UndoableEditListener undoListener);

	/**
	 * Sets up the button panel for the bottom of all panels to control moving
	 * through objects
	 * 
	 * @return the panel with the status, next, previous, and new object buttons
	 */
	protected JPanel getButtonPanel() {
		final JPanel buttonPanel = new JPanel();
		final GridBagConstraints gbc = new GridBagConstraints();
		if (nextButton != null) {
			nextButton.removeActionListener(nextListener);
			nextButton.addActionListener(nextListener);
		}

		if (prevButton != null) {
			prevButton.removeActionListener(prevListener);
			prevButton.addActionListener(prevListener);
		}

		if (deleteButton != null) {
			deleteButton.removeActionListener(deleteListener);
			deleteButton.addActionListener(deleteListener);
		}
		buttonPanel.setLayout(new GridBagLayout());
		int xcoord = 0;

		gbc.insets = new Insets(0, 8, 0, 0);
		gbc.gridx = xcoord++;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		gbc.gridwidth = 3;
		buttonPanel.add(statusLabel, gbc);

		gbc.gridx = xcoord = 0;
		gbc.gridy = 1;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridwidth = 1;
		gbc.weightx = 0.0;
		buttonPanel.add(deleteButton, gbc);

		gbc.gridx = ++xcoord;
		newButton.addActionListener(newListener);
		buttonPanel.add(newButton, gbc);

		/*
		 * gbc.gridx = ++xcoord; gbc.anchor = GridBagConstraints.WEST;
		 * //gbc.weightx = 0; gbc.weightx = 1.0; buttonPanel.add(findButton,
		 * gbc);
		 */

		gbc.gridx = ++xcoord;
		gbc.anchor = GridBagConstraints.EAST;
		buttonPanel.add(prevButton, gbc);

		gbc.gridx = ++xcoord;
		buttonPanel.add(nextButton, gbc);

		return buttonPanel;
	}

	/**
	 * Deletes the currently shown message from the database and shows the next
	 * message in the List
	 * 
	 * @return true if the deletion was successful, false otherwise
	 */
	public boolean deleteDisplayedObject() {
		try {
			List data = getDataList();
			Object object = data.get(getIndexOfDisplayed());
			// System.out.println(object);
			// make sure the object isn't null and the session knows about it
			if (object != null && getSession().contains(object)) {
				getSession().delete(object);
			}
			getDataList().remove(object);
			fillInText(getSession());
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * This calculates the minimum reasonable size for this panel
	 * 
	 * @return the minimum size this panel can be
	 */
	public Dimension getMinimumSize() {
		// System.out.println("You're looping " + this);
		// Thread.dumpStack();
		final int minHeight = this.getPreferredSize().height
				+ getButtonPanel().getPreferredSize().height;
		final int minWidth = Math.max(this.getPreferredSize().width,
				getButtonPanel().getPreferredSize().width);

		return new Dimension(minWidth, minHeight);
	}

	/**
	 * Performs all the saving operations necessary for exiting
	 * 
	 * @return true if the exit is ok to proceed, false otherwise
	 */
	public boolean exit() {
		if (getDataList() != null && isModified()) {
			String whichPanel = this.getName();
			// get the name without the word panel appended
			int n = JOptionPane.showConfirmDialog(null,
					"Warning: Data may not have been saved on panel: "
							+ whichPanel + ". \nEdits will"
							+ " be lost if file is not"
							+ " saved. Do you wish to save " + "now?");
			// 1 is no, 0 is yes and saved, 2 is cancel
			if (n < 2) {
				if (n == 0) {
					boolean isSaved = save();
					if (getSession() != null) {
						getSession().getTransaction().commit();
						getSession().beginTransaction();
					}
					return isSaved;
				} else {
					// user wants to exit without saving
					return true;
				}
			} else {
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * Fills in the status label with what number object we're on our of the
	 * total number of pieces of data we have
	 * 
	 * @param numberOn
	 *            The piece of data we're on
	 */
	protected abstract void setStatus(int numberOn);

	/**
	 * @return the status of this panel
	 */
	public String getStatusText() {
		return statusLabel.getText();
	}

	/**
	 * Saves the text that is currently present on the GUI
	 * 
	 * @return The object that was saved
	 * 
	 */
	public abstract Object saveText();

	/**
	 * Initializes the titlebar menu, key shortcuts, and the actions that occur
	 * when a menu item is selected
	 * 
	 * @param menuBar
	 *            the menubar to customize
	 */
	public void setMenuBar(final TabbedPanelMenu menuBar) {
		/* Start Menu code here */
		/*
		 * Initializes the titlebar menu, key shortcuts, and the actions that
		 * occur when a menu item is selected
		 */
		final JMenuItem save = menuBar.getSave();
		final JMenuItem cut = menuBar.getCut();
		final JMenuItem copy = menuBar.getCopy();
		final JMenuItem newMenu = menuBar.getNewMenu();
		final JMenuItem delete = menuBar.getDelete();
		undoMenu = menuBar.getUndo();
		redoMenu = menuBar.getRedo();
		paste = menuBar.getPaste();
		next = menuBar.getNext();
		prev = menuBar.getPrev();

		// final int keyMask = menuBar.keyMask;

		/* Add all the actionListeners to the components */
		save.addActionListener(saveListener);
		undoMenu.addActionListener(undoListener);
		redoMenu.addActionListener(redoListener);
		cut.addActionListener(cutListener);
		copy.addActionListener(copyListener);
		paste.addActionListener(pasteListener);
		next.addActionListener(nextListener);
		prev.addActionListener(prevListener);
		newMenu.addActionListener(newListener);
		delete.addActionListener(deleteListener);

		/* Set the menuItems we've implemented to be enabled */
		save.setEnabled(true);
		next.setEnabled(nextButton.isEnabled());
		prev.setEnabled(prevButton.isEnabled());
		cut.setEnabled(true);
		copy.setEnabled(true);
		paste.setEnabled(false);

		/* set the ones that we're not using invisible */
	}

	/**
	 * This function determines and sets the correct combination of dis/enabled
	 * buttons in the button panel and on the parent's menu
	 * 
	 */
	public void setButtonPermissions() {

		if (this.getIndexOfDisplayed() == this.maxIndex()) {
			nextButton.setEnabled(false);
			if (next != null) {
				next.setEnabled(false);
			}
			// newButton.setEnabled(true);
		} else {
			nextButton.setEnabled(true);
			if (next != null) {
				next.setEnabled(true);
			}
			// newButton.setEnabled(false);
		}
		if (getIndexOfDisplayed() <= 0) {
			prevButton.setEnabled(false);
			if (prev != null) {
				prev.setEnabled(false);
			}
			this.repaint();
		} else {
			prevButton.setEnabled(true);
			if (prev != null) {
				prev.setEnabled(true);
			}
		}
	}

	/**
	 * record which component has focus in order to be able to undo and redo, or
	 * insert text into the correct field
	 * 
	 * @param e
	 *            the focus event
	 * 
	 */
	public void focusGained(final FocusEvent e) {
		try {
			setHasFocus((JTextComponent) e.getSource());
		} catch (ClassCastException cce) {
			cce.printStackTrace();
		}
	}

	/**
	 * Does NOTHING.
	 * 
	 * @param e
	 *            the focus event
	 * 
	 */
	public void focusLost(FocusEvent e) {
		// empty
	}

	/**
	 * Method to get the findWindow associated with this BasePanel
	 * 
	 * @return this panel's FindWindow
	 */
	public abstract FindWindow getFindWindow();
	
}
