package airspaceGUI.base;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.JTextComponent;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Order;

import airspaceGUI.expansion.PanelUtils;

public abstract class SubPanel extends JPanel implements PanelUtils,
		FocusListener {
	private List dataList;
	private int indexOfDisplayed;
	private Session session;
	private JComponent hasFocus;

	/**
	 * if the user has made any changes
	 * */
	private boolean modified;

	/**
	 * if the system is loading an entity so that changes made to UI are not
	 * counted as user changes
	 * */
	private boolean pageLoading;

	/**
	 * This adds listeners for modifications to fields and values on the GUI so
	 * the modification state can be tracked
	 * 
	 * @param textFieldModificationListener
	 *            Listener to register modifications on text fields
	 * @param comboBoxModificationListener
	 *            Listener to register modifications on combo boxes
	 * @param checkBoxModificationListener
	 *            Listener to register modifications on check boxes
	 */
	protected abstract void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener);

	/**
	 * Adds Undoable edit listeners to the fields on the GUI so that changes can
	 * be undone.
	 * 
	 * @param undoListener
	 *            The Event Handler that handles undo events
	 */
	protected abstract void addUndoableEditListeners(
			UndoableEditListener undoListener);

	/**
	 * The number of data points that this panel has
	 * 
	 * @return the number of pieces of data that this panel has
	 */
	public int maxIndex() {
		return dataList.size() - 1;
	}

	/**
	 * Deletes the currently shown message from the database and shows the next
	 * message in the List
	 * 
	 * @return true if the deletion was successful, false otherwise
	 */
	public boolean deleteDisplayedObject() {
		Object objectToDelete = dataList.get(indexOfDisplayed);

		try {
			if (session.contains(objectToDelete)) {
				session.delete(objectToDelete);
				if (!session.getTransaction().isActive()) {
					session.beginTransaction();
				}
				session.getTransaction().commit();
				session.flush();
				session.beginTransaction();
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			// in case something goes wrong, we still want to remove it
			// from the list
			// System.out.println("deleting... " + objectToDelete);
			dataList.remove(objectToDelete);
			// the call below causes unnecessary popups, do not uncomment
			// unless this behavior is desired
			// clearText();
			fillInText(session);
			return true;
		}
	}

	/**
	 * @return Returns the dataList.
	 */
	protected List getDataList() {
		return dataList;
	}

	/**
	 * 
	 * @return the status of this panel
	 */
	public abstract String getStatusText();

	/**
	 * @return Returns the indexOfDisplayed.
	 */
	public int getIndexOfDisplayed() {
		// System.out.println("The index of displayed is " + indexOfDisplayed);
		return indexOfDisplayed;
	}

	/**
	 * @return Returns the session.
	 */
	protected Session getSession() {
		return session;
	}

	/**
	 * commits the current transaction and retains a new one
	 * */
	protected Transaction beginNewTransaction() {
		Session session = getSession();
		session.getTransaction().commit();

		setModified(false);
		return session.beginTransaction();
	}

	/**
	 * @return Returns the component that has focus.
	 */
	protected JComponent getHasFocus() {
		// System.out.println("the component that has focus is " + hasFocus);
		return hasFocus;
	}

	/**
	 * @param hasFocus
	 *            The component that should have focus.
	 */
	protected void setHasFocus(final JTextComponent hasFocus) {
		this.hasFocus = hasFocus;
	}

	/**
	 * @param dataList
	 *            The new list of data
	 */
	protected void setDataList(final List dataList) {
		this.dataList = dataList;
	}

	/**
	 * @param indexOfDisplayed
	 *            sets the index of the piece of data currently being displayed
	 */
	public void setIndexOfDisplayed(final int indexOfDisplayed) {
		this.indexOfDisplayed = indexOfDisplayed;
	}

	/**
	 * @param session
	 *            The session to get data out of
	 */
	protected void setSession(final Session session) {
		this.session = session;
	}

	/**
	 * Loads all the instances of the class in order of the orderby string from
	 * the database connected to the session and returns the entire list
	 * 
	 * @param session
	 *            a connected session to a database to load the objects from
	 * @param classToLoad
	 *            the class to load objects of
	 * @param orderBy
	 *            a string representation of the column the results should be
	 *            ordered by
	 * @return An array containing all instances of the class object that exist
	 *         in the database
	 */
	public final List load(final Session session, final Class classToLoad,
			final String orderBy) {
		if (session != null) {
			final ArrayList newerList = new ArrayList();
			if (this.session == null) {
				this.session = session;
			}

			if (!session.isOpen()) {
				this.session.beginTransaction();
			}
			List newList;

			newList = session.createCriteria(classToLoad).addOrder(
					Order.asc(orderBy)).list();

			return newList;
		}
		return new ArrayList();
	}

	/**
	 * Loads all instances of the class into the returned ArrayList
	 * 
	 * @param session
	 *            A session connected to the correct database
	 * @param classToLoad
	 *            The class to load all instances of
	 * @return An arrayList of all the instances of this class
	 */
	public final List load(final Session session, final Class classToLoad) {
		return load(session, classToLoad, "id");
	}

	/**
	 * Tests if this panel has been modified
	 * 
	 * @return true if this panel has been modified, false otherwise
	 */
	public boolean isModified() {
		System.out.println("calling modified: " + modified);
		return modified;
	}

	/**
	 * Tests if this panel has been modified
	 * 
	 * @param modified
	 *            the value of modified that isModified should return
	 */
	public void setModified(boolean modified) {
		System.out.println("modified: " + modified + " @" + this);

		this.modified = modified;
	}

	/**
	 * Saves the data back to the database
	 * 
	 * @param toSave
	 *            The data to save
	 * @return true if the save was successfuk, false otherwise
	 */
	public boolean save(List toSave) {
		if (session != null && (!session.isConnected() || !session.isOpen())) {
			session.beginTransaction();
		}

		if (session != null) {
			for (int i = 0; i < toSave.size(); i++) {
				// System.out.println(toSave.get(i));
				if (toSave.get(i) != null) {
					if (session.contains(toSave.get(i))) {
						// System.out.println("Updating in save");
						session.saveOrUpdate(toSave.get(i));
					} else {
						// System.out.println(i + " Saving transient");
						// System.out.println(toSave.get(i));
						session.save(toSave.get(i));
					}
				}
			}

			session.flush();
			beginNewTransaction();
		}
		setModified(false);
		return true;
	}

	/**
	 * persists the object
	 * */
	public boolean persist(Object obj) {
		if (session != null && (!session.isConnected() || !session.isOpen())) {
			session.beginTransaction();
		}

		if (session != null) {
			session.saveOrUpdate(obj);
			beginNewTransaction();
		}

		setModified(false);

		return true;
	}

	/**
	 * Performs all the saving operations necessary for exiting
	 * 
	 * @return true if the exit is ok to proceed, false otherwise
	 */
	public boolean exit() {
		if (dataList != null && isModified()) {
			final String whichPanel = this.getName();
			// get the name without the word panel appended
			final int n = JOptionPane.showConfirmDialog(null,
					"Warning: Data may not have been saved on panel: "
							+ whichPanel + ". \nEdits will"
							+ " be lost if file is not"
							+ " saved. Do you wish to save " + "now?");
			// 1 is no, 0 is yes and saved, 2 is cancel
			if (n < 2) {
				if (n == 0) {
					final boolean isSaved = save();
					if (session != null) {
						session.getTransaction().commit();
						session.beginTransaction();
					}
					return isSaved;
				} else {
					// user wants to exit without saving
					return true;
				}
			} else {
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * Saves the text that is currently present on the GUI
	 * 
	 * @return The object that was saved
	 * 
	 */
	public abstract Object saveText();

	/**
	 * clears the current panel and creates a new object associated with it
	 * */
	public abstract void createNewInstance();

	/**
	 * Makes the GUI display blank, so the user can enter an entirely new object
	 * 
	 */
	public abstract void clearPanel();

	/**
	 * record which component has focus in order to be able to insert variables
	 * into the field currently in focus
	 * 
	 * @param e
	 *            the focus event
	 * 
	 */
	public void focusGained(final FocusEvent e) {
		try {
			// System.out.println("focus gained");
			hasFocus = (JTextComponent) e.getSource();
		} catch (ClassCastException cce) {
			cce.printStackTrace();
		}
	}

	/**
	 * fills in the text fields with the properties of the object
	 * */
	public abstract void fillInText(Object object);

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
	 */
	public void focusLost(FocusEvent e) {
		// TODO Auto-generated method stub
	}

	/**
	 * if we are loading the page
	 * */
	public boolean isPageLoading() {
		return pageLoading;
	}

	/**
	 * sets the page loading
	 * */
	public void setPageLoading(boolean pageLoading) {
		this.pageLoading = pageLoading;
	}
}
