/**
 * 
 */
package airspaceGUI.expansion;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JTextArea;
import javax.swing.text.JTextComponent;

/**
 * This allows a normal JTextField implement auto-completion capabilities
 * @author srbennet
 * 
 */
public class AutoCompleteField extends JTextArea implements KeyListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /*
     * The list of possible objects the user may be trying to type in 
     * and should be searched for auto-complete possibilities
     */
    private transient List autoComplete;

    /**
     * Default Constructor, only adds the AutoComplete key listener, has 
     * no list to autocomplete with
     */
    public AutoCompleteField()
    {
        super();
        this.addKeyListener(this);
        // TODO Auto-generated constructor stub
    }

    /**
     * Constructor for the AutoCompleteField, initializes the textField and 
     * also registers the list to search when looking for possible completions
     * @param dataToSearch
     *            the data to search for auto-completion
     */
    public AutoCompleteField(List dataToSearch)
    {
        super();
        autoComplete = new ArrayList(dataToSearch);
        this.addKeyListener(this);
    }

    /**
     * A handler for keyTyped events in order to attempt to auto-complete the
     * string the user is typing.
     * 
     * @param e
     *            the key event that was generated by the user pressing a key
     */
    public void keyTyped(final KeyEvent e) {
        final JTextComponent writtenIn = (JTextComponent)e.getComponent();
        // the key that was typed
        final char keyCode = e.getKeyChar();

        // don;t handle enter, backspace, space, commat, or shift, meta or alt
        // events.
        // (we can't complete them, since they don't have an actual ascii
        // character
        // associated with them.)
        if (!e.isMetaDown() && keyCode != KeyEvent.VK_ENTER
                && keyCode != KeyEvent.VK_BACK_SPACE
                && (keyCode != KeyEvent.VK_SPACE || !(writtenIn.getText()
                        .endsWith(","))) && keyCode != KeyEvent.VK_COMMA
                && keyCode != KeyEvent.VK_SHIFT && keyCode != KeyEvent.VK_META
                && keyCode != KeyEvent.VK_ALT
                && writtenIn.getSelectionEnd() >= writtenIn.getText().length())
        {
            // need to take off the suggested text incase it doesn't match what
            // the user has typed
            final String text = writtenIn.getText().substring(0,
                    writtenIn.getSelectionStart());

            // get the pposition of the cursor, where the letter was added
            int caretPosition = writtenIn.getSelectionStart();

            // get the number of characters already typed so that we can
            // construct a string
            // of what has been typed by the user rather than what has been
            // inserted by the
            // auto-completion.
            int numAlreadyTyped; // the +2 is so that the
            // character string that
            // indicates the separation between entries is not
            // included in the string being completed.

            // remove any previously completed text, if it exists
            writtenIn.setText(writtenIn.getText().substring(0, caretPosition));

            // get the character sting the user has typed, so that it can be
            // completed correctly
            String prefix;
            if (!"".equals(text))
            {
                numAlreadyTyped = writtenIn.getSelectionStart()
                - (text.lastIndexOf(", ") + 2);
                prefix = text.substring(text.lastIndexOf(", ") + 2, text
                        .lastIndexOf(", ")
                        + 2 + numAlreadyTyped)
                        + keyCode;
            } else
            {
                numAlreadyTyped = 0;
                prefix = new String() + keyCode;
            }

            // create an arrayList that contains all the possibilities of what
            // the text could be
            // ArrayList, because of the possible expansion to put a popup box
            // to futher advise
            // the user of all the possibilities.
            final ArrayList possList = new ArrayList(getPossibilities(prefix));

            // make sure that there are some possibilities to complete
            if (!possList.isEmpty())
            {
                String suggestion = (String)possList.get(0);

                int i = 1;
                while (i < possList.size() && text.indexOf(suggestion) > -1)
                {
                    // only suggest an option if it isn't already in the string.
                    suggestion = (String)possList.get(i);
                    i++;
                }

                if (text.length() <= caretPosition)
                {
                    // add to the end of the list
                    writtenIn.setText(text
                            + suggestion.substring(numAlreadyTyped));
                } else
                {
                    // the user is typing in the middle.
                    writtenIn.setText(text.substring(0, caretPosition)
                            + suggestion.substring(numAlreadyTyped) + ", "
                            + text.substring(writtenIn.getCaretPosition()));
                }
                writtenIn.setCaretPosition(caretPosition++);
                writtenIn.select(caretPosition, caretPosition
                        + suggestion.substring(numAlreadyTyped + 1).length());
                //e.consume();
            }
        } else if (keyCode == KeyEvent.VK_ENTER)
        {
            // if they press the enter key, un-highlight the text and add the
            // seperator to the end.
            writtenIn.setText(writtenIn.getText());

            // consume this event so none of the other handlers get it.
            //e.consume();
        }
    }

    /**
     * Handler for key pressed events.
     * 
     * @param e
     *            the key event that was generated by the user pressing a key
     */
    public void keyPressed(final KeyEvent e) {
        //empty
    }

    /**
     * Handler for key released events.
     * 
     * @param e
     *            the key event that was generated by the user pressing a key
     */
    public void keyReleased(final KeyEvent e) {
        //empty
    }

    /**
     * Function to parse out all the possible things the user could be typing atLabel
     * this moment.
     * 
     * @param prefix
     *            the string that should be matched against to come up with
     *            possibilities
     * @return an ArrayList of all the possibilities that exist in autoComplete
     *         that might be what the user is trying to type
     */
    private List getPossibilities(final String prefix) {
        final ArrayList returnPossibilities = new ArrayList();
        final String uPrefix = prefix.substring(0, 1).toUpperCase() + prefix.substring(1, prefix.length());
        final String lPrefix = prefix.substring(0, 1).toLowerCase() + prefix.substring(1, prefix.length());

        for (int i = 0; i < autoComplete.size(); i++)
        {
            // need to check for null to protect against badly-formed arrays.
            if ((((String)autoComplete.get(i)) != null && ((String)autoComplete
                    .get(i)).startsWith(uPrefix))
                    || ((String)autoComplete.get(i)) != null
                    && ((String)autoComplete.get(i)).startsWith(lPrefix))
            {
                returnPossibilities.add(autoComplete.get(i));
            }
        }
        return returnPossibilities;
    }
}
